
# q-expansion of newform 7.9.b.b, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 7
weight = 9
poly_data = [51570, 0, 1016, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 3506, 0, 4], 201], [[0, 15454, 0, 8], 201], [[1016, 0, 2, 0], 67]]

hecke_ring_character_values = [[3, [-1, 0, 0, 0]]]
aps_data = [[8, 0, 0, 1], [0, -1, 0, 0], [0, 2, 1, 0], [357, 16, -1, 119], [-5542, 0, 0, -68], [0, -272, 17, 0], [0, -630, -18, 0], [0, 969, 102, 0], [227018, 0, 0, -5984], [-368254, 0, 0, -23324], [0, -2582, -526, 0], [1678818, 0, 0, 29716], [0, 5456, -782, 0], [1437018, 0, 0, 173656], [0, 16966, -3910, 0], [1687394, 0, 0, -379832], [0, 67847, 9826, 0], [0, -67474, 13243, 0], [17506778, 0, 0, -580704], [12475178, 0, 0, -1287818], [0, -49794, -29784, 0], [-20541814, 0, 0, -1054374], [0, 597295, 18428, 0], [0, -801618, 46716, 0], [0, 203798, 13006, 0], [0, 19312, -79849, 0], [0, -639098, -123556, 0], [-10494406, 0, 0, 628048], [-12154014, 0, 0, 17582896], [42991538, 0, 0, -10794014], [19820138, 0, 0, -6984264], [0, 4640675, 8260, 0], [76971554, 0, 0, -26261072], [0, -3834569, 277118, 0], [-486850174, 0, 0, -32774384], [535480458, 0, 0, 31041712], [0, 2619088, -246619, 0], [-559132006, 0, 0, -24735612], [0, -1441414, 176800, 0], [0, -8298500, -514267, 0], [-1064654470, 0, 0, 29583724], [0, 5858334, 382617, 0], [1463686442, 0, 0, 37975330], [-847985262, 0, 0, 42162346], [1173596258, 0, 0, -50103284], [0, 21053598, -1080996, 0], [-923415078, 0, 0, 129514840], [0, 6859568, 1632952, 0], [0, 15246673, -1951174, 0], [0, -14215400, -3252865, 0], [326082338, 0, 0, 39863776], [528279530, 0, 0, -304324496], [0, -57499846, 1492822, 0], [0, -19879443, -74664, 0], [0, -31479988, 2628268, 0], [300111338, 0, 0, 176687086], [0, 45902616, -1187367, 0], [0, -30007992, -8357676, 0], [-1300396606, 0, 0, -461193612], [5301977858, 0, 0, 65713312], [0, 37468597, 5758904, 0], [0, -34478584, -9814049, 0], [0, 48920271, -8144700, 0], [0, -50976436, 10699342, 0], [0, 56282684, 2785570, 0], [5895967778, 0, 0, -800635604], [178484538, 0, 0, 421573972], [3864974514, 0, 0, 179849698], [6791333978, 0, 0, 630294856], [0, -73195846, 1977967, 0], [0, -85202912, -6692968, 0], [-7504256374, 0, 0, 1098364576], [0, -14501392, -5767964, 0], [-24813824062, 0, 0, -434059044], [-8527218054, 0, 0, -537808124], [0, -15508114, -24958754, 0], [-6506131294, 0, 0, -387537764], [0, 349582494, 4493679, 0], [3322112018, 0, 0, 497244322], [0, -114190496, 25627262, 0], [0, -247619125, -39744860, 0], [25395765698, 0, 0, 2529454872], [-7417625878, 0, 0, -1183425080], [0, -195511866, -290190, 0], [0, -5920336, -16586798, 0], [-4464160966, 0, 0, 4214848828], [-413517214, 0, 0, -3133219544], [25077592338, 0, 0, -750070754], [0, -291750566, 40152827, 0], [12220926314, 0, 0, 2356259178], [0, 230258217, 31270038, 0], [0, 386168242, -54740374, 0], [28984976778, 0, 0, -442118864], [-98645183782, 0, 0, 1008536032], [-38403104550, 0, 0, -4823931356], [0, -610091544, 52791546, 0], [0, -330288886, -44156633, 0], [0, 743334264, -3128238, 0], [0, 709854975, 59524038, 0], [81731392898, 0, 0, 2620510152], [-31827726502, 0, 0, 2356566996], [27522917858, 0, 0, -8126615264], [0, 209146529, 134419510, 0], [39615074066, 0, 0, -3617343554], [29022209498, 0, 0, 4542893952], [0, -1386695508, -83053092, 0], [0, -231109781, 119786114, 0], [0, 1080773640, -57822372, 0], [53713956746, 0, 0, 5362915966], [0, 1433482226, 94459648, 0], [0, 610298608, -125468584, 0], [-243431631742, 0, 0, -209054424], [-107854289422, 0, 0, -3453566354], [0, 655204733, 137497324, 0], [-92407251382, 0, 0, 7600556502], [154625076818, 0, 0, 6098641642], [0, 1551277165, -42622162, 0], [0, -1044906258, 67996668, 0], [-151372882366, 0, 0, 10674999448], [-26079787270, 0, 0, 14568629824], [0, 1556414668, 10977359, 0], [147993728514, 0, 0, -10984557752], [0, 460195030, 72290237, 0], [188031665978, 0, 0, 1016997256], [0, -286908865, -326913740, 0], [-62334713278, 0, 0, -8669364200], [-27841944990, 0, 0, 12387616984], [0, -2310128250, 318219090, 0], [0, 869219690, -255683468, 0], [0, 891490234, -222377425, 0], [34678631066, 0, 0, -6556099044], [295521641354, 0, 0, -13790734832], [212837621802, 0, 0, -30656441360], [35113172418, 0, 0, 22017363256], [0, 4742032904, -275857538, 0], [0, 4198999762, 273314066, 0], [0, 1900856802, 216153249, 0], [0, 988097305, -351660778, 0], [0, -3637036780, -42035543, 0], [-423448492654, 0, 0, -24621446834], [0, -1899881751, 43435272, 0], [193183548482, 0, 0, -35407961420], [-369798055702, 0, 0, 31715228526], [339217053338, 0, 0, 32739975436], [0, -942574554, -413171757, 0], [0, -848739146, -8336428, 0], [0, 699406076, 322926031, 0], [0, -1303543916, 349879874, 0], [0, -4718319179, -412586872, 0], [-679145286742, 0, 0, 22586426626], [45970720034, 0, 0, -45682093932], [0, 1947417696, -497395332, 0], [507215441658, 0, 0, -15352876784], [0, -7969727974, 583697500, 0], [39387345594, 0, 0, 7155382348], [276819879338, 0, 0, -24913244048], [189083572266, 0, 0, -8869798874], [0, -8922245726, -17601238, 0], [0, -1240673430, 625584972, 0], [0, 3902309624, -992991743, 0], [818484719354, 0, 0, 18370919848], [648026042498, 0, 0, 35565947536], [-860784189046, 0, 0, 2042109888], [0, 8915310867, -681227094, 0], [321958750178, 0, 0, 54578017216], [0, 4513225282, -467599636, 0], [-1173749051382, 0, 0, 21395963122], [0, -8435447482, -74661059, 0]]
